#! /bin/bash

# 192.168.145.10

export DIR=/sys/kernel/debug/gfs2
export CLU=48b4b627:3c011797

sbstats () {
    cat  $DIR/$1/sbstats |
    awk '{
	s = 0
	for ( f = 3; f <= NF; f++ ) s = s + $f
	if ( NR > 1 ) print "gfs2.sbstats." $1 "." $2 s
    }'
}

glstats () {
    cat  $DIR/$1/glstats |
    awk '
    BEGIN {
	total = 0
        trans = 0
	inode = 0
	rgrp = 0
	meta = 0
	iopen = 0
	flock = 0
	quota = 0
	journal = 0
    }
	/G: n:1/  { trans++; total++ }
	/G: n:2/  { inode++; total++ }
	/G: n:3/  { rgrp++; total++ }
	/G: n:4/  { meta++; total++ }
	/G: n:5/  { iopen++; total++ }
	/G: n:6/  { flock++; total++ }
	/G: n:8/  { quota++; total++ }
	/G: n:9/  { journal++; total++ }
    END {
	print "glstats.total:" total
	print "glstats.trans:" trans
	print "glstats.inode:" inode
	print "glstats.rgrp:" rgrp
	print "glstats.meta:" meta
	print "glstats.iopen:" iopen
	print "glstats.flock:" flock
	print "glstats.quota:" quota
	print "glstats.journal:" journal
    }'
}

glocks () {
    cat  $DIR/$1/glocks |
    awk '

    function flag_is_set(str, flag) {
	s = substr(str, match(str, "f:") + 2);
	f = substr(s, 1, match(s,  " "));
	if ( match(f, flag) > 0 ) return 1;
	else return 0;
    }

    BEGIN {
	total = 0
        shared = 0
	unlocked = 0
	deferred = 0
	exclusive = 0
	glocks_flags_locked = 0
	glocks_flags_demote = 0
	glocks_flags_demote_pending = 0
	glocks_flags_demote_progress = 0
	glocks_flags_dirty = 0
	glocks_flags_log_flush = 0
	glocks_flags_invalidate = 0
	glocks_flags_reply_pending = 0
	glocks_flags_initial = 0
	glocks_flags_frozen = 0
	glocks_flags_queued = 0
	glocks_flags_object_attached = 0
	glocks_flags_blocking_request = 0
	glocks_flags_lru = 0
    }
	/G:  s:SH/  { shared++; total++ }
	/G:  s:UN/  { unlocked++; total++ }
	/G:  s:DF/  { deferred++; total++ }
	/G:  s:EX/  { exclusive++; total++ }
	/G:/	    {
		    glocks_flags_locked += flag_is_set($0, "l");
		    glocks_flags_demote += flag_is_set($0, "D");
		    glocks_flags_demote_pending += flag_is_set($0, "d");
		    glocks_flags_demote_progress += flag_is_set($0, "p");
		    glocks_flags_dirty += flag_is_set($0, "y");
		    glocks_flags_log_flush += flag_is_set($0, "f");
		    glocks_flags_invalidate += flag_is_set($0, "i");
		    glocks_flags_reply_pending += flag_is_set($0, "r");
		    glocks_flags_initial += flag_is_set($0, "I");
		    glocks_flags_frozen += flag_is_set($0, "f");
		    glocks_flags_queued += flag_is_set($0, "q");
		    glocks_flags_object_attached += flag_is_set($0, "o");
		    glocks_flags_blocking_request += flag_is_set($0, "b");
		    glocks_flags_lru += flag_is_set($0, "L");
 		    }
    END {
	print "glocks.total:" total
	print "glocks.shared:" shared
	print "glocks.unlocked:" unlocked
	print "glocks.deferred:" deferred
	print "glocks.exclusive:" exclusive
	print "glocks.flags.locked:" glocks_flags_locked
	print "glocks.flags.demote:" glocks_flags_demote
	print "glocks.flags.demotye_pending:" glocks_flags_demote_pending
	print "glocks.flags.demote_progress:" glocks_flags_demote_progress
	print "glocks.flags.dirty:" glocks_flags_dirty
	print "glocks.flags.log_flush:" glocks_flags_log_flush
	print "glocks.flags.invalidate:" glocks_flags_invalidate
	print "glocks.flags.reply_pending:" glocks_flags_reply_pending
	print "glocks.flags.initial:" glocks_flags_initial
	print "glocks.flags.frozen:" glocks_flags_frozen
	print "glocks.flags.queued:" glocks_flags_queued
	print "glocks.flags.object_attached:" glocks_flags_object_attached
	print "glocks.flags.blocking_request:" glocks_flags_blocking_request
	print "glocks.flags.lru:" glocks_flags_lru
    }'
}

holders () {
    cat  $DIR/$1/glocks |
    awk '

    function flag_is_set(str, flag) {
	s = substr(str, match(str, "f:") + 2);
	f = substr(s, 1, match(s,  " "));
	if ( match(f, flag) > 0 ) return 1;
	else return 0;
    }

    BEGIN {
	total = 0
        shared = 0
	unlocked = 0
	deferred = 0
	exclusive = 0
	holders_flags_async = 0
	holders_flags_any = 0
	holders_flags_no_cache = 0
	holders_flags_no_expire = 0
	holders_flags_exact = 0
	holders_flags_first = 0
	holders_flags_holder = 0
	holders_flags_priority = 0
	holders_flags_try = 0
	holders_flags_try_1cb = 0
	holders_flags_wait = 0
    }
    / H: s:SH/  { shared++; total++; }
    / H: s:UN/  { unlocked++; total++;}
    / H: s:DF/  { deferred++; total++ }
    / H: s:EX/  { exclusive++; total++ }
    / H:/	{
		holders_flags_async += flag_is_set($0, "a");
		holders_flags_any += flag_is_set($0, "A");
		holders_flags_no_cache += flag_is_set($0, "c");
		holders_flags_no_expire += flag_is_set($0, "e");
		holders_flags_exact += flag_is_set($0, "E");
		holders_flags_first += flag_is_set($0, "F");
		holders_flags_holder += flag_is_set($0, "H");
		holders_flags_priority += flag_is_set($0, "p");
		holders_flags_try += flag_is_set($0, "t");
		holders_flags_try_1cb += flag_is_set($0, "T");
		holders_flags_try_wait += flag_is_set($0, "W");
		}
    END {
	print "holders.total:" total
	print "holders.shared:" shared
	print "holders.unlocked:" unlocked
	print "holders.deferred:" deferred
	print "holders.exclusive:" exclusive

	print "holders.flags.async:" holders_flags_async
	print "holders.flags.any:" holders_flags_any
	print "holders.flags.no_cache:" holders_flags_no_cache
	print "holders.flags.no_expire:" holders_flags_no_expire
	print "holders.flags.exact:" holders_flags_exact
	print "holders.flags.first:" holders_flags_first
	print "holders.flags.holder:" holders_flags_holder
	print "holders.flags.priority:" holders_flags_priority
	print "holders.flags.try:" holders_flags_try
	print "holders.flags.try_1cb:" holders_flags_try_1cb
	print "holders.flags.wait:" holders_flags_wait
    }'
}

echo "---glocks---"
glocks $CLU
echo "---holders---"
holders $CLU
echo "---sbstats---"
sbstats $CLU | tac
echo "---glstats---"
glstats $CLU

